#
# Copyright (C) 2021-2022 by AyiinXd@Github, < https://github.com/AyiinXd >.
#
# This file is part of < https://github.com/AyiinXd/AyiinMusicBot > project,
# and is released under the "GNU v3.0 License Agreement".
# Please see < https://github.com/AyiinXd/AyiinMusicBot/blob/master/LICENSE >
#
# All rights reserved.

import os
import re
import textwrap

import aiofiles
import aiohttp
from PIL import (
    Image,
    ImageDraw,
    ImageEnhance,
    ImageFilter,
    ImageFont,
    ImageOps,
)

try:
    from youtubesearchpython.__future__ import VideosSearch
except ImportError:
    print("'youtubesearchpython' tidak terinstall\nMungkin beberapa modul tidak akan bisa digunakan.")
    pass


class Thumbnail(object):
    def changeImageSize(self, maxWidth, maxHeight, image):
        widthRatio = maxWidth / image.size[0]
        heightRatio = maxHeight / image.size[1]
        newWidth = int(widthRatio * image.size[0])
        newHeight = int(heightRatio * image.size[1])
        newImage = image.resize((newWidth, newHeight))
        return newImage


    async def gen_thumb(
        self, 
        client,
        videoid,
        file_path,
        cache,
        cache_thumb,
        fonts,
        fonts2,
    ):
        # cache = f"cache/{videoid}.png"
        # cache_thumb = f"cache/thumb{videoid}.png"
        # fonts = "assets/font.ttf"
        # fonts2 = "assets/font2.ttf"
        if not os.path.isdir(file_path):
            os.makedirs(file_path)

        if os.path.isfile(cache):
            return cache

        url = f"https://www.youtube.com/watch?v={videoid}"
        try:
            results = VideosSearch(url, limit=1)
            for result in (await results.next())["result"]:
                try:
                    title = result["title"]
                    title = re.sub("\\W+", " ", title)
                    title = title.title()
                except BaseException:
                    title = "Unsupported Title"
                try:
                    duration = result["duration"]
                except BaseException:
                    duration = "Unknown Mins"
                thumbnail = result["thumbnails"][0]["url"].split("?")[0]
                try:
                    views = result["viewCount"]["short"]
                except BaseException:
                    views = "Unknown Views"
                try:
                    channel = result["channel"]["name"]
                except BaseException:
                    channel = "Unknown Channel"

            async with aiohttp.ClientSession() as session:
                async with session.get(thumbnail) as resp:
                    if resp.status == 200:
                        f = await aiofiles.open(
                            cache_thumb, mode="wb"
                        )
                        await f.write(await resp.read())
                        await f.close()

            youtube = Image.open(cache_thumb)
            image1 = self.changeImageSize(1280, 720, youtube)
            image2 = image1.convert("RGBA")
            background = image2.filter(filter=ImageFilter.BoxBlur(30))
            enhancer = ImageEnhance.Brightness(background)
            background = enhancer.enhance(0.6)
            Xcenter = youtube.width / 2
            Ycenter = youtube.height / 2
            x1 = Xcenter - 250
            y1 = Ycenter - 250
            x2 = Xcenter + 250
            y2 = Ycenter + 250
            logo = youtube.crop((x1, y1, x2, y2))
            logo.thumbnail((520, 520), Image.ANTIALIAS)
            logo = ImageOps.expand(logo, border=15, fill="white")
            background.paste(logo, (50, 100))
            draw = ImageDraw.Draw(background)
            font = ImageFont.truetype(fonts2, 40)
            font2 = ImageFont.truetype(fonts2, 70)
            arial = ImageFont.truetype(fonts2, 30)
            name_font = ImageFont.truetype(fonts, 35)
            para = textwrap.wrap(title, width=32)
            me = await client.get_me()
            uname = me.username
            music_name = f'{uname.upper()} UBOT' if uname else 'AYIIN UBOT'
            j = 0
            draw.text(
                (30, 5), f"{music_name}", fill="white", font=name_font
            )
            draw.text(
                (600, 150),
                "NOW PLAYING",
                fill="white",
                stroke_width=2,
                stroke_fill="white",
                font=font2,
            )
            for line in para:
                if j == 1:
                    j += 1
                    draw.text(
                        (600, 340),
                        f"{line}",
                        fill="white",
                        stroke_width=1,
                        stroke_fill="white",
                        font=font,
                    )
                if j == 0:
                    j += 1
                    draw.text(
                        (600, 280),
                        f"{line}",
                        fill="white",
                        stroke_width=1,
                        stroke_fill="white",
                        font=font,
                    )

            draw.text(
                (600, 450),
                f"Views : {views[:23]}",
                (255, 255, 255),
                font=arial,
            )
            draw.text(
                (600, 500),
                f"Duration : {duration[:23]} Mins",
                (255, 255, 255),
                font=arial,
            )
            draw.text(
                (600, 550),
                f"Channel : {channel}",
                (255, 255, 255),
                font=arial,
            )
            try:
                os.remove(cache_thumb)
            except BaseException:
                pass
            background.save(cache)
            return cache
        except BaseException:
            results = VideosSearch(url, limit=1)
            for result in results.result()["result"]:
                thumbnail = result["thumbnails"][0]["url"].split("?")[0]
            return thumbnail



# py - Ayiin
# Copyright (C) 2022-2023 @AyiinXd
#
# This file is a part of < https://github.com/AyiinXd/pyAyiin >
# PLease read the GNU Affero General Public License in
# <https://www.github.com/AyiinXd/pyAyiin/blob/main/LICENSE/>.
#
# FROM py-Ayiin <https://github.com/AyiinXd/pyAyiin>
# t.me/AyiinChat & t.me/AyiinSupport


# ========================×========================
#            Jangan Hapus Credit Ngentod
# ========================×========================

from fipper.enums import ParseMode
from fipper.types import (
    InlineKeyboardButton,
    InlineKeyboardMarkup,
    InlineQueryResultArticle,
    InlineQueryResultPhoto,
    InlineQueryResultVideo,
    InputTextMessageContent,
)

from ..config import Var
from ..pyrogram.toolbot import ToolBot


class InlineBot(Var, ToolBot):
    async def inline_pmpermit(self, user_id, ids):
        from pyAyiin.dB.pmpermit import setting_pmpermit, get_media_pmermit, get_message_pmermit
        
        button = [
            [
                InlineKeyboardButton(
                    text='• Approve •',
                    callback_data=f'terima_{user_id}xd{ids}',
                ),
                InlineKeyboardButton(
                    text='• Disapprove •',
                    callback_data=f'tolak_{user_id}xd{ids}',
                ),
            ],
            [
                InlineKeyboardButton(
                    text='• Close •',
                    callback_data=f'close',
                ),
            ]
        ]
        pm_message = await get_message_pmermit(user_id)
        media_message = await get_media_pmermit(user_id)
        endsw = (".mp4", ".gif")
        if media_message: # Jika database HELP_PIC tidak kosong
            if media_message.endswith(endsw): # jika format video atau gif 
                pm_results = [
                    (
                        InlineQueryResultVideo(
                            video_url=media_message,
                            thumb_url=media_message,
                            title="PmPermit",
                            description="inline AyiinUbot.",
                            caption=pm_message,
                            reply_markup=InlineKeyboardMarkup(button),
                        )
                    )
                ]
            else: # Format foto
                pm_results = [
                    (
                        InlineQueryResultPhoto(
                            photo_url=media_message,
                            title="PmPermit",
                            description="inline AyiinUbot.",
                            caption=pm_message,
                            reply_markup=InlineKeyboardMarkup(button),
                        )
                    )
                ]
        else: # Jika database HELP_PIC kosong akan menjadi teks biasa tanpa foto atau video
            pm_results = [
                (
                    InlineQueryResultArticle(
                        title='PmPermit Ayiin Ubot!',
                        reply_markup=InlineKeyboardMarkup(button),
                        input_message_content=InputTextMessageContent(pm_message),
                    )
                )
            ]
        
        return pm_results
    
    async def inline_help(self, output):
        from pyAyiin import CMD_HELP
        from pyAyiin.dB.variable import get_var
        
        HELP_PIC = await get_var('HELP_PIC')
        endsw = (".mp4", ".gif")
        if HELP_PIC is not None: # Jika database HELP_PIC tidak kosong
            if HELP_PIC.endswith(endsw): # jika format video atau gif 
                results = [
                    (
                        InlineQueryResultVideo(
                            video_url=HELP_PIC,
                            thumb_url=HELP_PIC,
                            title="Help",
                            description="inline AyiinUbot.",
                            caption=output,
                            reply_markup=InlineKeyboardMarkup(
                                self.HelpXd(0, CMD_HELP, "xd")
                            ),
                        )
                    )
                ]
            else: # Format foto
                results = [
                    (
                        InlineQueryResultPhoto(
                            photo_url=HELP_PIC,
                            title="Help",
                            description="inline AyiinUbot.",
                            caption=output,
                            reply_markup=InlineKeyboardMarkup(
                                self.HelpXd(0, CMD_HELP, "xd")
                            ),
                        )
                    )
                ]
        else: # Jika database HELP_PIC kosong akan menjadi teks biasa tanpa foto atau video
            results = [
                (
                    InlineQueryResultArticle(
                        title="Help Ayiin Ubot!",
                        reply_markup=InlineKeyboardMarkup(
                                self.HelpXd(0, CMD_HELP, "xd")
                            ),
                        input_message_content=InputTextMessageContent(output),
                    )
                )
            ]
        return results
    
    async def inline_alive(self, output):
        from pyAyiin.dB.variable import get_var
        
        alive_pic = await get_var('ALIVE_PIC')
        buttons = [
            [
                InlineKeyboardButton("•• Group ••", url='https://t.me/AyiinChats'),
                InlineKeyboardButton("•• Channel ••", url='https://t.me/AyiinChannel'),
            ]
        ]
        endsw = (".mp4", ".gif")
        if alive_pic is not None: # Jika database alive tidak kosong
            if alive_pic.endswith(endsw): # jika format video atau gif
                results = [
                    (
                        InlineQueryResultVideo(
                            video_url=alive_pic,
                            thumb_url=alive_pic,
                            title="Alive",
                            description="inline AyiinUbot.",
                            caption=output,
                            reply_markup=InlineKeyboardMarkup(buttons),
                        )
                    )
                ]
            else:
                results = [
                    (
                        InlineQueryResultPhoto(
                            photo_url=alive_pic,
                            title="Alive",
                            description="inline AyiinUbot.",
                            caption=output,
                            reply_markup=InlineKeyboardMarkup(buttons),
                        )
                    )
                ]
        else:
            results = [
                (
                    InlineQueryResultArticle(
                        title="Alive Ayiin Ubot!",
                        reply_markup=InlineKeyboardMarkup(buttons),
                        input_message_content=InputTextMessageContent(output),
                    )
                )
            ]
        return results




from ._core import mongodb


captchadb = mongodb.captcha
solved_captcha_db = mongodb.solved_captcha
captcha_cachedb = mongodb.captcha_cache
welcomedb = mongodb.welcome_text


async def is_captcha_on(chat_id: int) -> bool:
    chat = await captchadb.find_one({"chat_id": chat_id})
    if not chat:
        return True
    return False


async def captcha_on(chat_id: int):
    is_captcha = await is_captcha_on(chat_id)
    if is_captcha:
        return
    return await captchadb.delete_one({"chat_id": chat_id})


async def captcha_off(chat_id: int):
    is_captcha = await is_captcha_on(chat_id)
    if not is_captcha:
        return
    return await captchadb.insert_one({"chat_id": chat_id})


async def has_solved_captcha_once(chat_id: int, user_id: int):
    has_solved = await solved_captcha_db.find_one(
        {"chat_id": chat_id, "user_id": user_id}
    )
    return bool(has_solved)


async def save_captcha_solved(chat_id: int, user_id: int):
    return await solved_captcha_db.update_one(
        {"chat_id": chat_id},
        {"$set": {"user_id": user_id}},
        upsert=True,
    )


async def get_welcome(chat_id: int) -> str:
    text = await welcomedb.find_one({"chat_id": chat_id})
    if not text:
        return ""
    return text["text"]


async def set_welcome(chat_id: int, text: str):
    return await welcomedb.update_one(
        {"chat_id": chat_id}, {"$set": {"text": text}}, upsert=True
    )


async def del_welcome(chat_id: int):
    return await welcomedb.delete_one({"chat_id": chat_id})


async def update_captcha_cache(captcha_dict):
    pickle = obj_to_str(captcha_dict)
    await captcha_cachedb.delete_one({"captcha": "cache"})
    if not pickle:
        return
    await captcha_cachedb.update_one(
        {"captcha": "cache"},
        {"$set": {"pickled": pickle}},
        upsert=True,
    )


async def get_captcha_cache():
    cache = await captcha_cachedb.find_one({"captcha": "cache"})
    if not cache:
        return []
    return str_to_obj(cache["pickled"])



import yaml

from os import listdir, path

from ._core import mongodb


langdb = mongodb.langs

langm = {}
languages = {}


# language
async def get_lang(user_id: int) -> str:
    mode = langm.get(user_id)
    if not mode:
        lang = await langdb.find_one({"user_id": user_id})
        if not lang:
            langm[user_id] = "en"
            return "en"
        langm[user_id] = lang["lang"]
        return lang["lang"]
    return mode


async def set_lang(user_id: int, lang: str):
    langm[user_id] = lang
    await langdb.update_one(
        {"user_id": user_id}, {"$set": {"lang": lang}}, upsert=True
    )


def get_string(lang: str):
    return languages[lang]


'''
strings_folder = path.join(path.dirname(path.realpath(__file__)), "pyAyiin/langs")
for filename in listdir(strings_folder):
    if filename.endswith(".yml"):
        language_name = filename[:-4]
        languages[language_name] = yaml.safe_load(
            open(path.join(strings_folder, filename), encoding="utf8")
        )
'''
